import numpy as np
import matplotlib.pyplot as plt
import toolbox_sr1
import control

def f_fwd(x,u):
    """ Modellgleichung Traktor
    """
    
    # Parameter
    V  = 5 # [m/s]
    L1 = 6 # [m]
    L2 = 4 # [m]

    # ODE
    xdot = np.array([
        [V/L1*np.tan(u)],
        [V/L2*np.cos(x[0] - x[1] - np.pi/2)]
    ])

    return np.squeeze(xdot)

def f_bwd(x,u):
    """ Modellgleichung Traktor
    """
    
    # Parameter
    V  = -5 # [m/s]
    L1 = 6 # [m]
    L2 = 4 # [m]

    # ODE
    xdot = np.array([
        [V/L1*np.tan(u)],
        [V/L2*np.cos(x[0] - x[1] - np.pi/2)]
    ])

    return np.squeeze(xdot)

def g(x,u):
    """ Ausgangsgleichung Traktor
    """
    return x[1]

# Aufgabe 2a)
# Linearisieren Sie das System zunächst um den Gleichgewichtspunkt x_ss = [0; 0], u_ss = 0 .
# Berechnen Sie die Matrizen A, B, C, D des linearisierten Zustandsraummodells.

# Parameter
V = 5.0
L1 = 6.0
L2 = 4.0

# Gleichgewichtszustand
x_ss = np.zeros((2,1))
u_ss = 0.0

# Zustandsraummodell
A = np.matrix([
    [0,0],
    [V/L2*(np.cos(x_ss[0]-x_ss[1])), -V/L2*(np.cos(x_ss[0]-x_ss[1]))]])
B = np.matrix([
    [V/(L1*np.cos(u_ss)**2)],
    [0.0]])
C = np.matrix([[0, 1]])
D = np.matrix([[0]])

# Aufgabe 2b)
# Erstellen Sie nun mithilfe des python-control-Pakets das Zustandsraummodell. Nutzen Sie hierzu
# den Befehl ss(...).

ss_fwd = control.ss(A,B,C,D)

# Aufgabe 2c) 
# Überprüfen Sie das erhaltene System auf Stabilität.

pole_fwd = control.pole(ss_fwd)
print(pole_fwd)

# plt.show()

# Aufgabe 2e) Wiederholen Sie die S

# Rückwärtsgeschwindigkiet
V = -5

# Zustandsraummodell
A = np.matrix([
    [0,0],
    [V/L2*(np.cos(x_ss[0]-x_ss[1])), -V/L2*(np.cos(x_ss[0]-x_ss[1]))]])
B = np.matrix([
    [V/(L1*np.cos(u_ss)**2)],
    [0.0]])

ss_bwd = control.ss(A,B,C,D)
pole_bwd = control.pole(ss_bwd)
print(pole_bwd)

T = np.linspace(0,10.0, int(10.0/0.01))
U = 0.1*np.ones((1000,))
[T1, Y_fwd, X_fwd] = control.forced_response(ss_fwd, T = T, X0=x_ss, U = U)
[T2, Y_bwd, X_bwd] = control.forced_response(ss_bwd, T = T, X0=x_ss, U = U)

[X_fwd_nl,Y_fwd_nl,T_nl] = toolbox_sr1.nlsim(f_fwd, [0.0, 0.0], U, 0.01, g)
[X_bwd_nl,Y_bwd_nl,T_nl] = toolbox_sr1.nlsim(f_bwd, [0.0, 0.0], U, 0.01, g)

plt.figure()
plt.plot(T1, X_fwd[0,:],color='blue')
plt.plot(T2, X_fwd[1,:],color='red')
plt.plot(T_nl, X_fwd_nl[:,0],color ='blue',linestyle='--')
plt.plot(T_nl, X_fwd_nl[:,1],color='red',linestyle='--')
plt.legend(['beta','gamma','beta_nl','gamma_nl'])
plt.grid(True)

plt.figure()
plt.plot(T1, X_bwd[0,:],color='blue')
plt.plot(T2, X_bwd[1,:],color='red')
plt.plot(T_nl, X_bwd_nl[:,0],color ='blue',linestyle='--')
plt.plot(T_nl, X_bwd_nl[:,1],color ='red',linestyle='--')
plt.ylim((-4, 4))
plt.grid(True)
plt.legend(['beta','gamma','beta_nl','gamma_nl'])
plt.show()
